@@ -27,6 +27,8 @@ import shutil
 import tempfile
 import jinja2
 import time
+import threading
+import json
 from productmd.composeinfo import ComposeInfo
 from kobo.conf import PyConfigParser
 
@@ -35,13 +37,16 @@ from odcs.server import conf, log, db
 from odcs.server import comps
 from odcs.server.models import Compose
 from odcs.common.types import (
-    PungiSourceType, COMPOSE_RESULTS, MULTILIB_METHODS,
-    INVERSE_PUNGI_SOURCE_TYPE_NAMES, COMPOSE_FLAGS)
+    PungiSourceType,
+    COMPOSE_RESULTS,
+    MULTILIB_METHODS,
+    INVERSE_PUNGI_SOURCE_TYPE_NAMES,
+    COMPOSE_FLAGS,
+)
 from odcs.server.utils import makedirs, clone_repo, copytree
 
 
 class BasePungiConfig(object):
-
     def __init__(self):
         self.pungi_timeout = conf.pungi_timeout
 
@@ -53,13 +58,12 @@ class BasePungiConfig(object):
         :param str cfg: Configuration to write.
         """
         with open(path, "w") as f:
-            log.info("Writing %s configuration to %s.",
-                     os.path.basename(path), path)
+            log.info("Writing %s configuration to %s.", os.path.basename(path), path)
             f.write(cfg)
 
     def write_config_files(self, topdir):
         """Write configuration into files"""
-        raise NotImplementedError('Concrete config object must implement.')
+        raise NotImplementedError("Concrete config object must implement.")
 
     def validate(self, topdir, compose_dir):
         """Validate configuration. Raises an exception of error found."""
@@ -67,10 +71,10 @@ class BasePungiConfig(object):
 
 
 class RawPungiConfig(BasePungiConfig):
-
-    def __init__(self, compose_source):
+    def __init__(self, compose):
         super(RawPungiConfig, self).__init__()
-        source_name, source_hash = compose_source.split("#")
+        self.compose = compose
+        source_name, source_hash = compose.source.split("#")
 
         url_data = copy.deepcopy(conf.raw_config_urls[source_name])
         # Do not override commit hash by hash from ODCS client if it is
@@ -79,9 +83,27 @@ class RawPungiConfig(BasePungiConfig):
             url_data["commit"] = source_hash
 
         self.pungi_timeout = url_data.get("pungi_timeout", conf.pungi_timeout)
+        self.raw_config_wrapper_conf_path = url_data.get(
+            "raw_config_wrapper", conf.raw_config_wrapper_conf_path
+        )
         self.pungi_cfg = url_data
         self.pungi_koji_args = conf.raw_config_pungi_koji_args.get(
-            source_name, conf.pungi_koji_args)
+            source_name, conf.pungi_koji_args
+        )
+
+    def apply_raw_config_wrapper_overrides(self, cfg_path):
+        try:
+            with open(cfg_path) as fd:
+                raw_config_wrapper = jinja2.Template(fd.read())
+            with open(cfg_path, "w") as fd:
+                fd.write(raw_config_wrapper.render(compose=self.compose.json()))
+        except Exception as e:
+            log.exception(
+                "Failed to render raw_config_wrapper template {!r}: {}".format(
+                    cfg_path, str(e)
+                )
+            )
+            raise
 
     def write_config_files(self, topdir):
         """Write raw config files
@@ -93,17 +115,18 @@ class RawPungiConfig(BasePungiConfig):
         # the raw_config wrapper as real "pungi.conf".
         # The reason is that wrapper config can import raw_config
         # and override some variables.
-        if conf.raw_config_wrapper_conf_path:
+        if self.raw_config_wrapper_conf_path:
             main_cfg_path = os.path.join(topdir, "raw_config.conf")
-            shutil.copy2(conf.raw_config_wrapper_conf_path,
-                         os.path.join(topdir, "pungi.conf"))
+            shutil.copy2(
+                self.raw_config_wrapper_conf_path, os.path.join(topdir, "pungi.conf")
+            )
+            self.apply_raw_config_wrapper_overrides(os.path.join(topdir, "pungi.conf"))
         else:
             main_cfg_path = os.path.join(topdir, "pungi.conf")
 
         # Clone the git repo with raw_config pungi config files.
         repo_dir = os.path.join(topdir, "raw_config_repo")
-        clone_repo(self.pungi_cfg["url"], repo_dir,
-                   commit=self.pungi_cfg["commit"])
+        clone_repo(self.pungi_cfg["url"], repo_dir, commit=self.pungi_cfg["commit"])
 
         # If the 'path' is defined, copy only the files form the 'path'
         # to topdir.
@@ -112,10 +135,47 @@ class RawPungiConfig(BasePungiConfig):
 
         copytree(repo_dir, topdir)
 
-        # Create the "pungi.conf" from config_filename.
-        config_path = os.path.join(topdir, self.pungi_cfg["config_filename"])
-        if config_path != main_cfg_path:
-            shutil.copy2(config_path, main_cfg_path)
+        # If pungi_config_dump is defined, it means that we are regenerating
+        # already built production compose and this should use the very same
+        # Pungi config file as the original one.
+        # We still want to use the latest raw_config_wrapper, because this
+        # is used to configure some important variables which can change
+        # in the future ODCS deployment - like "koji_profile" or "runroot_method".
+        # We also want to be able to respin the compose with the very same
+        # configuration but include one new package build to fix last-minute
+        # issues with the compose - this new build will also be set using the
+        # raw_config_wrapper.
+        if self.compose.pungi_config_dump:
+            # Load the config dump.
+            pungi_config_dump = json.loads(self.compose.pungi_config_dump)
+
+            # If raw_config_wrapper is used, update the pungi_config_dump according
+            # to it.
+            if self.raw_config_wrapper_conf_path:
+                # At first create empty "raw_config.conf" file so "raw_config_wrapper.conf"
+                # can import it and can be loaded by PyConfigParser.
+                with open(main_cfg_path, "w") as fd:
+                    fd.write("# empty config file")
+
+                # Load the raw_config_wrapper and update pungi_config_dump.
+                raw_config_wrapper = PyConfigParser()
+                raw_config_wrapper.load_from_file(main_cfg_path)
+                pungi_config_dump.update(raw_config_wrapper)
+
+            # Create the pungi.json
+            with open(os.path.join(topdir, "pungi.json"), "w") as fd:
+                fd.write(json.dumps(pungi_config_dump))
+        else:
+            # Create the "pungi.conf" from config_filename.
+            config_path = os.path.join(topdir, self.pungi_cfg["config_filename"])
+            if config_path != main_cfg_path:
+                shutil.copy2(config_path, main_cfg_path)
+            c = PyConfigParser()
+
+            # Create pungi.json.
+            c.load_from_file(os.path.join(topdir, "pungi.conf"))
+            with open(os.path.join(topdir, "pungi.json"), "w") as fd:
+                fd.write(json.dumps(c))
 
     def validate(self, topdir, compose_dir):
         if not conf.pungi_config_validate:
@@ -125,15 +185,19 @@ class RawPungiConfig(BasePungiConfig):
         # Apply global schema override.
         if conf.raw_config_schema_override:
             pungi_config_validate_cmd += [
-                "--schema-override", conf.raw_config_schema_override]
+                "--schema-override",
+                conf.raw_config_schema_override,
+            ]
 
         # Apply raw_config specific schema override.
         if "schema_override" in self.pungi_cfg:
             pungi_config_validate_cmd += [
-                "--schema-override", self.pungi_cfg["schema_override"]]
+                "--schema-override",
+                self.pungi_cfg["schema_override"],
+            ]
 
         # Add raw_config configuration file to validate.
-        pungi_config_validate_cmd.append(os.path.join(topdir, "pungi.conf"))
+        pungi_config_validate_cmd.append(os.path.join(topdir, "pungi.json"))
 
         # Run the pungi-config-validate. The execute_cmd raises an exception
         # if config is invalid.
@@ -142,15 +206,30 @@ class RawPungiConfig(BasePungiConfig):
         with open(log_out_path, "w") as log_out:
             with open(log_err_path, "w") as log_err:
                 odcs.server.utils.execute_cmd(
-                    pungi_config_validate_cmd, stdout=log_out, stderr=log_err)
+                    pungi_config_validate_cmd, stdout=log_out, stderr=log_err
+                )
 
 
 class PungiConfig(BasePungiConfig):
-    def __init__(self, release_name, release_version, source_type, source,
-                 packages=None, arches=None, sigkeys=None, results=0,
-                 multilib_arches=None, multilib_method=0, builds=None,
-                 flags=0, lookaside_repos=None, modular_koji_tags=None,
-                 module_defaults_url=None):
+    def __init__(
+        self,
+        release_name,
+        release_version,
+        source_type,
+        source,
+        packages=None,
+        arches=None,
+        sigkeys=None,
+        results=0,
+        multilib_arches=None,
+        multilib_method=0,
+        builds=None,
+        flags=0,
+        lookaside_repos=None,
+        modular_koji_tags=None,
+        module_defaults_url=None,
+        scratch_modules=None,
+    ):
         super(PungiConfig, self).__init__()
         self.release_name = release_name
         self.release_version = release_version
@@ -167,6 +246,7 @@ class PungiConfig(BasePungiConfig):
             self.arches = conf.arches
         self.packages = packages or []
         self.builds = builds or []
+        self.scratch_modules = scratch_modules.split(" ") if scratch_modules else []
 
         # Store results as list of strings, so it can be used by jinja2
         # templates.
@@ -186,8 +266,12 @@ class PungiConfig(BasePungiConfig):
             self.bootable = True
 
         if source_type == PungiSourceType.KOJI_TAG:
-            self.koji_module_tags = modular_koji_tags.split(" ") if modular_koji_tags else []
-            self.module_defaults_url = module_defaults_url.split(" ") if module_defaults_url else []
+            self.koji_module_tags = (
+                modular_koji_tags.split(" ") if modular_koji_tags else []
+            )
+            self.module_defaults_url = (
+                module_defaults_url.split(" ") if module_defaults_url else []
+            )
             self.koji_tag = source
             self.gather_source = "comps"
             if self.koji_module_tags:
@@ -198,14 +282,19 @@ class PungiConfig(BasePungiConfig):
             self.koji_tag = None
             self.gather_source = "module"
             self.gather_method = "nodeps"
-            self.module_defaults_url = module_defaults_url.split(" ") if module_defaults_url else []
+            self.module_defaults_url = (
+                module_defaults_url.split(" ") if module_defaults_url else []
+            )
 
             if self.packages:
-                raise ValueError("Exact packages cannot be set for MODULE "
-                                 "source type.")
-        elif source_type in [PungiSourceType.BUILD,
-                             PungiSourceType.PUNGI_COMPOSE,
-                             PungiSourceType.REPO]:
+                raise ValueError(
+                    "Exact packages cannot be set for MODULE " "source type."
+                )
+        elif source_type in [
+            PungiSourceType.BUILD,
+            PungiSourceType.PUNGI_COMPOSE,
+            PungiSourceType.REPO,
+        ]:
             self.gather_source = "comps"
             self.gather_method = "deps"
             self.koji_tag = None
@@ -232,14 +321,20 @@ class PungiConfig(BasePungiConfig):
     @property
     def pkgset_source(self):
         if self.source_type == PungiSourceType.REPO:
-            return 'repos'
-        return 'koji'
+            return "repos"
+        return "koji"
+
+    @property
+    def mbs_api_url(self):
+        return os.path.join(conf.mbs_url, "1")
 
     def get_comps_config(self):
         if self.source_type == PungiSourceType.MODULE:
             return ""
         odcs_comps = comps.Comps()
-        odcs_group = comps.Group('odcs-group', 'odcs-group', 'ODCS compose default group')
+        odcs_group = comps.Group(
+            "odcs-group", "odcs-group", "ODCS compose default group"
+        )
         for package in self.packages:
             odcs_group.add_package(comps.Package(package))
         odcs_comps.add_group(odcs_group)
@@ -249,7 +344,9 @@ class PungiConfig(BasePungiConfig):
 
     def get_variants_config(self):
         odcs_product = comps.Product()
-        tmp_variant = comps.Variant('Temporary', 'Temporary', 'variant', self.source_type)
+        tmp_variant = comps.Variant(
+            "Temporary", "Temporary", "variant", self.source_type
+        )
         for arch in self.arches:
             tmp_variant.add_arch(comps.Arch(arch))
         if self.source_type == PungiSourceType.MODULE:
@@ -257,7 +354,11 @@ class PungiConfig(BasePungiConfig):
                 tmp_variant.add_module(comps.Module(module))
         elif self.source_type == PungiSourceType.KOJI_TAG:
             if self.packages:
-                tmp_variant.add_group(comps.Group('odcs-group', 'odcs-group', 'ODCS compose default group'))
+                tmp_variant.add_group(
+                    comps.Group(
+                        "odcs-group", "odcs-group", "ODCS compose default group"
+                    )
+                )
             if self.koji_module_tags:
                 tmp_variant.add_module(comps.Module("*"))
 
@@ -270,11 +371,16 @@ class PungiConfig(BasePungiConfig):
         try:
             with open(conf.pungi_conf_path) as fd:
                 template = jinja2.Template(fd.read())
-            return template.render(config=self)
+            c_data = template.render(config=self)
+            c = PyConfigParser()
+            c.load_from_string(c_data)
+            return c
         except Exception as e:
             log.exception(
-                "Failed to render pungi conf template {!r}: {}".format(conf.pungi_conf_path,
-                                                                       str(e)))
+                "Failed to render pungi conf template {!r}: {}".format(
+                    conf.pungi_conf_path, str(e)
+                )
+            )
 
     def write_config_files(self, topdir):
         """
@@ -293,11 +399,44 @@ class PungiConfig(BasePungiConfig):
         log.debug("Comps.xml:")
         log.debug("%s", comps_cfg)
 
-        self._write(os.path.join(topdir, "pungi.conf"), main_cfg)
+        self._write(os.path.join(topdir, "pungi.json"), json.dumps(main_cfg))
         self._write(os.path.join(topdir, "variants.xml"), variants_cfg)
         self._write(os.path.join(topdir, "comps.xml"), comps_cfg)
 
 
+class ReadComposeIdThread(threading.Thread):
+    def __init__(self, compose):
+        threading.Thread.__init__(self)
+        self._stop_event = threading.Event()
+        self.compose = compose
+
+    def stop(self):
+        self._stop_event.set()
+
+    def run(self):
+        p = os.path.join(
+            self.compose.toplevel_dir, "work", "global", "composeinfo-base.json"
+        )
+        while not self._stop_event.is_set():
+            time.sleep(1)
+
+            # File does not exist yet.
+            if not os.path.exists(p):
+                continue
+
+            ci = ComposeInfo()
+            try:
+                ci.load(p)
+            except Exception:
+                # This should happen only if file exists, but the data is not
+                # written yet.
+                continue
+
+            self.compose.pungi_compose_id = ci.compose.id
+            db.session.commit()
+            break
+
+
 class Pungi(object):
     def __init__(self, compose_id, pungi_cfg, koji_event=None, old_compose=None):
         self.compose_id = compose_id
@@ -312,7 +451,7 @@ class Pungi(object):
         """
         self.pungi_cfg.write_config_files(topdir)
 
-    def get_pungi_cmd(self, conf_topdir, targetdir, compose_dir=None):
+    def get_pungi_cmd(self, conf_topdir, compose, compose_dir=None):
         """
         Returns list with pungi command line arguments needed to generate
         the compose.
@@ -326,20 +465,41 @@ class Pungi(object):
         """
         pungi_cmd = [
             conf.pungi_koji,
-            "--config=%s" % os.path.join(conf_topdir, "pungi.conf"),
+            "--config=%s" % os.path.join(conf_topdir, "pungi.json"),
+            "--no-latest-link",
         ]
 
         if compose_dir:
             pungi_cmd.append("--compose-dir=%s" % compose_dir)
         else:
-            pungi_cmd.append("--target-dir=%s" % targetdir)
+            pungi_cmd.append("--target-dir=%s" % compose.target_dir)
 
         if isinstance(self.pungi_cfg, RawPungiConfig):
             pungi_cmd += self.pungi_cfg.pungi_koji_args
         elif isinstance(self.pungi_cfg, PungiConfig):
             pungi_cmd += conf.pungi_koji_args
         else:
-            raise RuntimeError('Unknown pungi config type to handle.')
+            raise RuntimeError("Unknown pungi config type to handle.")
+
+        compose_type_to_arg = {
+            "test": "--test",
+            "ci": "--ci",
+            "nightly": "--nightly",
+            "production": "--production",
+        }
+        compose_type = compose.compose_type or "test"
+
+        # Add compose_type arg to pungi_cmd only if it's not set already
+        # directly in the configuration.
+        if not set(pungi_cmd).intersection(set(compose_type_to_arg.values())):
+            # For unknown compose_type, fallback to --test to be safe.
+            pungi_cmd.append(compose_type_to_arg.get(compose_type, "--test"))
+
+        if compose.label:
+            pungi_cmd.append("--label=%s" % str(compose.label))
+        if compose.parent_pungi_compose_ids:
+            for compose_id in compose.parent_pungi_compose_ids.split(" "):
+                pungi_cmd.append("--parent-compose-id=%s" % str(compose_id))
 
         if self.koji_event:
             pungi_cmd += ["--koji-event", str(self.koji_event)]
@@ -347,15 +507,18 @@ class Pungi(object):
             pungi_cmd += ["--old-composes", self.old_compose]
         return pungi_cmd
 
-    def _prepare_compose_dir(self, compose, conf_topdir):
+    def _prepare_compose_dir(self, compose, conf):
         """
         Creates the compose directory and returns the full path to it.
         """
         compose_date = time.strftime("%Y%m%d", time.localtime())
         makedirs(compose.toplevel_dir)
 
-        conf = PyConfigParser()
-        conf.load_from_file(os.path.join(conf_topdir, "pungi.conf"))
+        # If Compose Tracking Service is configured in the config file,
+        # we skip the ComposeInfo creation completely and instead let
+        # the Pungi to ask CTS to generate unique ComposeInfo.
+        if "cts_url" in conf and "cts_keytab" in conf:
+            return compose.toplevel_dir
 
         ci = ComposeInfo()
         ci.release.name = conf["release_name"]
@@ -378,7 +541,8 @@ class Pungi(object):
         while True:
             ci.compose.id = ci.create_compose_id()
             existing_compose = Compose.query.filter(
-                Compose.pungi_compose_id == ci.compose.id).first()
+                Compose.pungi_compose_id == ci.compose.id
+            ).first()
             if not existing_compose:
                 break
             ci.compose.respin += 1
@@ -397,34 +561,52 @@ class Pungi(object):
         Runs local Pungi compose.
         """
         td = None
+        compose_id_thread = None
         try:
             td = tempfile.mkdtemp()
             self._write_cfgs(td)
-            compose_dir = self._prepare_compose_dir(compose, td)
+
+            # Load pungi configuration file.
+            with open(os.path.join(td, "pungi.json")) as fd:
+                pungi_conf = json.load(fd)
+
+            compose_dir = self._prepare_compose_dir(compose, pungi_conf)
             self.pungi_cfg.validate(td, compose_dir)
-            pungi_cmd = self.get_pungi_cmd(td, compose.target_dir, compose_dir)
+            pungi_cmd = self.get_pungi_cmd(td, compose, compose_dir)
 
             # Commit the session to ensure that all the `compose` changes are
             # stored database before executing the compose and are not just
             # cached locally in the SQLAlchemy.
             db.session.commit()
 
+            # If Compose Tracking Service is configured in the config file,
+            # we need to get the Compose ID from Pungi in separate thread.
+            if "cts_url" in pungi_conf and "cts_keytab" in pungi_conf:
+                compose_id_thread = ReadComposeIdThread(compose)
+                compose_id_thread.start()
+
             log_out_path = os.path.join(compose_dir, "pungi-stdout.log")
             log_err_path = os.path.join(compose_dir, "pungi-stderr.log")
 
             with open(log_out_path, "w") as log_out:
                 with open(log_err_path, "w") as log_err:
                     odcs.server.utils.execute_cmd(
-                        pungi_cmd, cwd=td, timeout=self.pungi_cfg.pungi_timeout,
-                        stdout=log_out, stderr=log_err)
+                        pungi_cmd,
+                        cwd=td,
+                        timeout=self.pungi_cfg.pungi_timeout,
+                        stdout=log_out,
+                        stderr=log_err,
+                    )
         finally:
+            if compose_id_thread:
+                compose_id_thread.stop()
             try:
                 if td is not None:
                     shutil.rmtree(td)
             except Exception as e:
                 log.warning(
-                    "Failed to remove temporary directory {!r}: {}".format(
-                        td, str(e)))
+                    "Failed to remove temporary directory {!r}: {}".format(td, str(e))
+                )
 
     def run(self, compose):
         """
@@ -449,8 +631,7 @@ class PungiLogs(object):
         toplevel_dir = self.compose.toplevel_dir
         if not toplevel_dir:
             return None
-        return os.path.join(
-            toplevel_dir, "logs", "global", "pungi.global.log")
+        return os.path.join(toplevel_dir, "logs", "global", "pungi.global.log")
 
     @property
     def config_dump_path(self):
@@ -460,8 +641,7 @@ class PungiLogs(object):
         toplevel_dir = self.compose.toplevel_dir
         if not toplevel_dir:
             return None
-        return os.path.join(
-            toplevel_dir, "logs", "global", "config-dump.global.log")
+        return os.path.join(toplevel_dir, "logs", "global", "config-dump.global.log")
 
     def _get_global_log_errors(self):
         """
@@ -488,7 +668,7 @@ class PungiLogs(object):
                         continue
                     if error:
                         errors.append(error)
-                    error = line[idx + len("[ERROR   ] "):]
+                    error = line[idx + len("[ERROR   ] ") :]
         except IOError:
             pass
         return errors
@@ -513,8 +693,7 @@ class PungiLogs(object):
                 break
 
         if self.compose.on_default_target_dir:
-            errors = errors.replace(
-                conf.target_dir, conf.target_dir_url)
+            errors = errors.replace(conf.target_dir, conf.target_dir_url)
         return errors
 
     def get_config_dump(self):
