@@ -30,7 +30,6 @@ from odcs.server.utils import execute_cmd
 
 
 class TestUtilsExecuteCmd(unittest.TestCase):
-
     def setUp(self):
         super(TestUtilsExecuteCmd, self).setUp()
 
@@ -39,8 +38,7 @@ class TestUtilsExecuteCmd(unittest.TestCase):
 
     def test_execute_cmd_timeout_called(self):
         start_time = time.time()
-        with six.assertRaisesRegex(
-                self, RuntimeError, 'Compose has taken more time.*'):
+        with six.assertRaisesRegex(self, RuntimeError, "Compose has taken more time.*"):
             execute_cmd(["/usr/bin/sleep", "5"], timeout=1)
         stop_time = time.time()
 
